local summonFrame = FrameHelper:CreateFrame("Summon list")
local joinedGroup = false
WokrahReport = {}
summonFrame:Hide()

function WokrahReport:BuildFrame()
    if Wokrah["IS_PLAYER_WARLOCK"] then
        FrameHelper:RemoveAllClickAbleButtons(summonFrame)

        for k,v in pairs(WokrahSummonList) do 
            FrameHelper:CreateClickableButton(summonFrame, v, WokrahReport.OnPreClick, WokrahReport.OnRemoveClick)
        end

        if getn(WokrahSummonList) > 0 then
            summonFrame:Show()
        else
            summonFrame:Hide()
        end
    end
end

function WokrahReport:SetSummonList(newWokrahSummonList)
    WokrahSummonList = {}

    if string.gmatch(newWokrahSummonList, "!") then -- Delimiter is '!'
        local strList = { strsplit("!", newWokrahSummonList) }

        for k,v in pairs(strList) do
            tinsert(WokrahSummonList, v)
        end
    else -- We only have 1 person in the list
        tinsert(WokrahSummonList, newWokrahSummonList)
    end

    WokrahReport:BuildFrame()
end

function WokrahReport:GetSummonListAsPlainText()
    local plainTextList = nil
                
    for k,v in pairs(WokrahSummonList) do
        if plainTextList then
            plainTextList = plainTextList..'!'..v
        else
            plainTextList = v
        end
    end
    
    return plainTextList
end

-- Can be called by the comms class to remove the player
function WokrahReport:RemovePlayerFromList(playerName)
    local removedPlayer = false

    for k,v in pairs(WokrahSummonList) do 
        if v == playerName then
            tremove(WokrahSummonList, k)
            removedPlayer = true
        end
    end

    if removedPlayer then
        WokrahReport:BuildFrame()
    end

    return removedPlayer
end

-- An entry point for macro usage, you can create a macro that can do "/run WokrahAddon:SummonFirstQueuedPlayer()"
function WokrahAddon:SummonFirstQueuedPlayer()
    local firstQueuedPlayer = summonFrame.buttons[0]

    if firstQueuedPlayerButton then
        WokrahReport.OnPreClick(firstQueuedPlayerButton, nil, nil)
    end
end

-- As this is an event handler I need to use the '.' annotation in the func name instead of a ':'
function WokrahReport.OnPreClick(source, button, down)
    if not InCombatLockdown() and UnitPower("player") > 300 then
        local playerName = source:GetText()
        -- Create a macro which will target the player we want and cast the spell ritual of summoning
        if WokrahReport:RemovePlayerFromList(playerName) then 
            source:SetAttribute("type1", "macro")
            source:SetAttribute("macrotext1", "/target "..playerName.."\n/cast Ritual of Summoning")

            WokrahComms:SummonPlayerBroadcast(playerName)
        end
    end
end

function WokrahReport.OnRemoveClick(source, button, down)
    local playerName = source.HiddenText
    WokrahReport:RemovePlayerFromList(playerName) 
end

-- Only add the player to the list if we don't already have them
local function AddPlayerToList(playerName)
    if Wokrah["IS_PLAYER_WARLOCK"] then
        for k,v in pairs(WokrahSummonList) do 
            if v == playerName then
                return false
            end
        end

        tinsert(WokrahSummonList, playerName)
        WokrahReport:BuildFrame()

        return true
    end

    return false
end

function WokrahReport:ProcessMessage(event, text, playerName, languageName, channelName, playerName2, specialFlags, zoneChannelID, channelIndex, channelBaseName, unused, lineID, guid, bnSenderID, isMobile, isSubtitle, hideSenderInLetterbox, supressRaidIcons)
    -- If the message starts with 123 and we are of class warlock then we want to summon the person that said 123
    if string.StartsWith(text, "123") then
        playerName = strsplit("-", playerName)
        AddPlayerToList(playerName)
    end
end

-- If we join a group then we want to check if there are other people in this group with this addon and sync their summon list
-- If we have left the group we can clear the list
function WokrahReport:RosterUpdate(...)
    if not joinedGroup and (IsInGroup() or IsInRaid()) then
        WokrahComms:SyncWithGroup(nil)
        joinedGroup = true
    elseif joinedGroup and not (IsInGroup() or IsInRaid()) then
        WokrahSummonList = {}
        joinedGroup = false
        WokrahReport:BuildFrame()
    end
end

function WokrahReport:OnLogin()
    if WokrahSummonList == nil then
        WokrahSummonList = {}
    end

    if IsInGroup() or IsInRaid() then
        WokrahComms:SyncWithGroup()
        joinedGroup = true
    else
        -- As we're no longer in a group we can clear the list upon login
        WokrahSummonList = {}
    end
    
    WokrahReport:BuildFrame()
end